@@ -6,45 +6,50 @@ use warnings;
 use 5.008003;
 
 use Term::Choose::Constants qw( WIDTH_CURSOR );
+use Term::Choose::Screen    qw( clear_screen );
 use Term::Choose::Util      qw( get_term_width );
 
 
 sub new {
     my ( $class, $self ) = @_;
     bless $self, $class;
-    my $count_cells = $self->{row_count} * $self->{col_count};
+    my $count_cells = $self->{data_row_count} * $self->{col_count};
     if ( $self->{threshold} && $self->{threshold} < $count_cells ) {
+        print clear_screen;
         print "\rComputing: ";
-        $self->{times} = 3;
         if ( $count_cells / $self->{threshold} > 50 ) {
-            $self->{type} = 'multi';
-            $self->{total} = $self->{row_count};
+            $self->{merge_progress_bars} = 0;
+            $self->{total} = $self->{data_row_count};
         }
         else {
-            $self->{type} = 'single';
-            $self->{total} = $self->{row_count} * $self->{times};
+            $self->{merge_progress_bars} = 1;
+            $self->{total} = $self->{data_row_count} * $self->{count_progress_bars};
         }
     }
+    else {
+        $self->{count_progress_bars} = 0;
+    }
     return $self;
 }
 
 
 sub set_progress_bar {
     my ( $self ) = @_;
-    if (! $self->{type} ) {
+    if ( ! $self->{count_progress_bars} ) {
         return;
     }
     my $term_w = get_term_width();
     if ( $^O ne 'MSWin32' && $^O ne 'cygwin' ) {
         $term_w += WIDTH_CURSOR;
     }
-    if ( $self->{type} eq 'multi' ) {
-        $self->{fmt} = "\rComputing: (" . $self->{times}-- . ") %3d%% [%s]";
-    }
-    else {
+    if ( $self->{merge_progress_bars} ) {
         $self->{fmt} = "\rComputing: %3d%% [%s]";
     }
+    else {
+        $self->{fmt} = "\rComputing: (" . $self->{count_progress_bars} . ") %3d%% [%s]";
+    }
     if ( $term_w < 25 ) {
+        # fmt
         $self->{short_print} = 1;
     }
     else {
@@ -53,14 +58,14 @@ sub set_progress_bar {
     $self->{bar_w} = $term_w - length( sprintf $self->{fmt}, 100, '' ) + 1; # +1: lenght("\r") == 1
     $self->{step} = int( $self->{total} / $self->{bar_w} || 1 );
     my $count;
-    if ( $self->{type} eq 'multi' ) {
-        $count = 0;
-        $self->{next_update} = $self->{step};
-    }
-    else {
+    if ( $self->{merge_progress_bars} ) {
         $count = $self->{so_far} || 0;
         $self->{next_update} ||= $self->{step};
     }
+    else {
+        $count = 0;
+        $self->{next_update} = $self->{step};
+    }
     return $count;
 }
 
@@ -80,12 +85,13 @@ sub update_progress_bar {
 
 sub last_update_progress_bar {
     my ( $self, $count ) = @_;
-    if ( $self->{times} < 1 ||  $self->{type} eq 'multi' ) {
-        $self->update_progress_bar( $self->{total} );
-    }
-    else {
+    if ( $self->{count_progress_bars} &&  $self->{merge_progress_bars} ) {
         $self->{so_far} = $count;
     }
+    else {
+        $self->update_progress_bar( $self->{total} );
+    }
+    $self->{count_progress_bars}--;
 }
 
 
